/*
 * $Id: parse_url.h 4 2006-10-26 17:54:40Z jos $
 *
 * parse_url -- simple url parsing routine
 *
 * parse_url() is taken from the source code of
 * Dissonance - Nintendo DS Internet Radio Client
 * Copyright (C) 2006 David Brady
 *
 * Modifications:
 * Copyright (C) 2006 Jos Hendriks (jos@circuitdb.com)
 *
 */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

//Parses a URL. Return 0 if everything is OK and -1 if the parsing failed.
//Allocates memory to prot, host, and file. Assumes null.
int parse_url(char** oprot, char** ohost, char** ofile, int* oport, const char* address) {
	int i;
	int begin = 0;
	int next = 0;
	char* temp;
	char* prot = *oprot;
	char* host = *ohost;
	char* file = *ofile;
	int   port = *oport;
	
	//Try to get the protocol
	for(i = begin; i < strlen(address); i++) {
		if(address[i] == ':' && address[i+1] == '/' && address[i+2] == '/') {
			next = i+3;
			prot = malloc((next-begin)+1);
			strncpy(prot, address+begin, next-begin);
			prot[next-begin] = '\0';
			begin = next;
			break;
		}
	}
	if(!prot || !strlen(prot)) { // if no protocol is found assume http
		temp = "http://";
		prot = malloc(strlen(temp)+1);
		strncpy(prot, temp, strlen(temp));
		prot[strlen(temp)+1] = '\0';
	}
	
	//Try to get the host
	for(i = begin; i <= strlen(address); i++) {
		// hostnames may not begin with ':'
		if((address[i] == ':') && i==begin) {
			return -1;
		}
		//remove leading '/'
		if(address[i]=='/' && i==begin) {
			begin = i+1;
		}else{
		    if(address[i] == ':' || address[i]=='/' || i==strlen(address)) {
			next = i;
			host = malloc((next-begin)+1);
			strncpy(host, address+begin, next-begin);
			host[next-begin] = '\0';
			begin = next;
			break;
		    }
		}
	}
	//Try to get the port
	if (address[begin] == ':'){ //If there is no ':' then no port is specified
		for(i = begin; i < strlen(address); i++) {
			if(address[i] == ':') {
				for(next = i+1; 
					address[next] != '/' && next < strlen(address);
					next++);
				temp = malloc((next-begin)+1);
				strncpy(temp, address+begin+1, next-begin);
				temp[next-begin] = '\0';
				port = atoi(temp);
				begin = next;
				free(temp);
				break;
			}
		}
	}
	if(!port)
		port = 40;

	//gets file
	next = strlen(address);
	file = malloc((next-begin)+1);
	strncpy(file, address+begin, next-begin);
	file[next-begin] = '\0';
	if(!file || !strlen(file)) {
		free(file);
		file = malloc(2);
		file[0] = '/';
		file[1] = '\0';
	}
	*ohost = host;
	*ofile = file;
	*oport = port;
	*oprot = prot;
	return 0;
}
